@@ -19,7 +19,7 @@
 
 #include <iostream>
 #include <fstream>
-#include <getopt.h>
+#include "getopt_long.h"
 
 #include <lttoolbox/fst_processor.h>
 
@@ -42,12 +42,12 @@ using namespace std;
 
 
 void help(char *name) {
-  cerr<<"USAGE:\n";
-  cerr<<name<<" --trainwrd stopwords words n left right corpus model [--weightexp w]\nOR\n";
-  cerr<<name<<" --trainlch stopwords lexchoices n left right corpus wordmodel dic bildic model [--weightexp w]\nOR\n";
-  cerr<<name<<" --lextor model dic left right [--debug] [--weightexp w]\n\n";
-  //cerr<<name<<" --lextortl stopwords words tlmodel dic bildic left right [--debug] [--weightexp w]\n\n"; 
-  cerr<<"ARGUMENTS: \n"
+  wcerr<<"USAGE:\n";
+  wcerr<<name<<" --trainwrd stopwords words n left right corpus model [--weightexp w]\nOR\n";
+  wcerr<<name<<" --trainlch stopwords lexchoices n left right corpus wordmodel dic bildic model [--weightexp w]\nOR\n";
+  wcerr<<name<<" --lextor model dic left right [--debug] [--weightexp w]\n\n";
+  //wcerr<<name<<" --lextortl stopwords words tlmodel dic bildic left right [--debug] [--weightexp w]\n\n"; 
+  wcerr<<"ARGUMENTS: \n"
       <<"   --trainwrd|-t: Train word co-occurrences model.\n"
       <<"   Required parameters:\n"
       <<"      stopwords: file containing a list of stop words. Stop words are ignored\n"
@@ -94,14 +94,12 @@ void help(char *name) {
       <<"   --debug|-d: Show debug information while operating\n"
       <<"   --help|-h: Show this help\n"
       <<"   --version|-v: Show version information\n\n";
-  cerr<<"Reads from standard input and writes to standard output\n";
+  wcerr<<"Reads from standard input and writes to standard output\n";
 }
 
 int main(int argc, char* argv[]) {
   int c;
-#if HAVE_GETOPT_LONG
   int option_index=0;
-#endif
   int mode=-1;
 
   //Parameters for the "trainwrd" or the "trainlch" mode option
@@ -125,10 +123,9 @@ int main(int argc, char* argv[]) {
 
   LexTor::debug=false;
 
-  //cerr<<"LOCALE: "<<setlocale(LC_ALL,"")<<"\n";
+  //wcerr<<"LOCALE: "<<setlocale(LC_ALL,"")<<"\n";
 
   while (true) {
-#if HAVE_GETOPT_LONG
     static struct option long_options[] =
       {
 	{"trainwrd",  required_argument, 0, 't'},
@@ -143,9 +140,6 @@ int main(int argc, char* argv[]) {
       };
 
     c=getopt_long(argc, argv, "t:r:l:e:w:dhv",long_options, &option_index);
-#else
-    c=getopt(argc, argv, "t:r:l:e:w:dhv");
-#endif
     if (c==-1)
       break;
       
@@ -370,7 +364,7 @@ int main(int argc, char* argv[]) {
       exit(EXIT_FAILURE);
     }
     if (bildic_file=="") {
-      cerr<<"Error: No bilingual dictionary was provided\n";
+      wcerr<<"Error: No bilingual dictionary was provided\n";
       help(argv[0]);
       exit(EXIT_FAILURE);
     }
