@@ -21,7 +21,8 @@
 #include <string>
 #include <sys/types.h>
 #include <unistd.h>
-#include <getopt.h>
+#include "getopt_long.h"
+#include <apertium/string_utils.h>
 
 #include <apertium/hmm.h>
 #include <apertium/tagger_data_hmm.h>
@@ -38,17 +39,17 @@ TTag eos; //End-of-sentence tag
 
 void check_file(FILE *f, const string& path) {
   if (!f) {
-    cerr<<"Error: cannot open file '"<<path<<"'\n";
+    wcerr<<"Error: cannot open file '"<<path.c_str()<<"'\n";
     exit(EXIT_FAILURE);
   }
 }
 
 void help(char *name) {
-  cerr<<"Forbid and enforce rules are applied to the given HMM parameters\n\n";
-  cerr<<"USAGE:\n";
-  cerr<<name<<" --filein filein.prob --fileout fileout.prob --tsxfile file.tsx\n\n";
+  wcerr<<"Forbid and enforce rules are applied to the given HMM parameters\n\n";
+  wcerr<<"USAGE:\n";
+  wcerr<<name<<" --filein filein.prob --fileout fileout.prob --tsxfile file.tsx\n\n";
 
-  cerr<<"ARGUMENTS: \n"
+  wcerr<<"ARGUMENTS: \n"
       <<"   --filein|-i: To specify the file with the HMM parameter to process\n\n"
       <<"   --fileout|-o: To specify the file to which the HMM will be written\n\n"
       <<"   --tsxfile|-x: File containing the rules to apply\n\n"
@@ -61,17 +62,14 @@ int main(int argc, char* argv[]) {
   string filetsx="";
 
   int c;
-#if HAVE_GETOPT_LONG
   int option_index=0;
-#endif
 
-  cerr<<"Command line: ";
+  wcerr<<"Command line: ";
   for(int i=0; i<argc; i++)
-    cerr<<argv[i]<<" ";
-  cerr<<"\n";
+    wcerr<<argv[i]<<" ";
+  wcerr<<"\n";
 
   while (true) {
-#if HAVE_GETOPT_LONG
     static struct option long_options[] =
       {
 	{"filein",    required_argument, 0, 'i'},
@@ -81,9 +79,6 @@ int main(int argc, char* argv[]) {
       };
 
     c=getopt_long(argc, argv, "i:o:x:hv",long_options, &option_index);
-#else
-    c=getopt(argc, argv, "i:o:x:hv");
-#endif
     if (c==-1)
       break;
       
@@ -101,7 +96,7 @@ int main(int argc, char* argv[]) {
       filetsx=optarg;
       break;
     case 'v':
-      cerr<<"LICENSE:\n\n"
+      wcerr<<"LICENSE:\n\n"
 	  <<"   Copyright (C) 2006 Felipe Snchez Martnez\n\n"
 	  <<"   This program is free software; you can redistribute it and/or\n"
 	  <<"   modify it under the terms of the GNU General Public License as\n"
@@ -125,19 +120,19 @@ int main(int argc, char* argv[]) {
 
   //Now we check the command line arguments
   if (filein=="") {
-    cerr<<"Error: You did not provide an input file (.prob). Use --filein to do that\n";
+    wcerr<<"Error: You did not provide an input file (.prob). Use --filein to do that\n";
     help(argv[0]);
     exit(EXIT_FAILURE);
   }
 
   if (fileout=="") {
-    cerr<<"Error: You did not provide an output file (.prob). Use --fileout to do that\n";
+    wcerr<<"Error: You did not provide an output file (.prob). Use --fileout to do that\n";
     help(argv[0]);
     exit(EXIT_FAILURE);
   }
 
   if (filetsx=="") {
-    cerr<<"Error: You did not provide a tagger definition file (.tsx). Use --filetsx to do that\n";
+    wcerr<<"Error: You did not provide a tagger definition file (.tsx). Use --filetsx to do that\n";
     help(argv[0]);
     exit(EXIT_FAILURE);
   }
@@ -147,15 +142,15 @@ int main(int argc, char* argv[]) {
   fin=fopen(filein.c_str(), "rb");
   check_file(fin, filein);
 
-  cerr<<"Reading apertium-tagger data from file '"<<filein<<"' ... "<<flush;
+  wcerr<<"Reading apertium-tagger data from file '"<<filein<<"' ... "<<flush;
   tagger_data_hmm.read(fin);
   fclose(fin);
-  cerr<<"done.\n";
+  wcerr<<"done.\n";
 
-  cerr<<"Reading apertium-tagger definition from file '"<<filetsx<<"' ... "<<flush;
+  wcerr<<"Reading apertium-tagger definition from file '"<<filetsx<<"' ... "<<flush;
   TSXReader treader;
   treader.read(filetsx);
-  cerr<<"done.\n";
+  wcerr<<"done.\n";
   
   tagger_data_hmm.setForbidRules(treader.getTaggerData().getForbidRules());
   tagger_data_hmm.setEnforceRules(treader.getTaggerData().getEnforceRules());
@@ -166,8 +161,8 @@ int main(int argc, char* argv[]) {
 
   fout=fopen(fileout.c_str(), "wb");
   check_file(fout, fileout);
-  cerr<<"Writing apertium-tagger data to file '"<<fileout<<"' ... "<<flush;
+  wcerr<<"Writing apertium-tagger data to file '"<<fileout<<"' ... "<<flush;
   hmm.serialise(fout);
   fclose(fout);
-  cerr<<"done.\n";
+  wcerr<<"done.\n";
 }
