@@ -24,7 +24,7 @@
 #include <sys/stat.h>
 #include <unistd.h>
 #include <apertium/string_utils.h>
-#include <getopt.h>
+#include "getopt_long.h"
 #ifdef _MSC_VER
 #include <io.h>
 #include <fcntl.h>
@@ -35,25 +35,25 @@ using namespace std;
 
 void message(char *progname)
 {
-  cerr << "USAGE: " << basename(progname) << " trules preproc biltrans [input [output]]" << endl;
-  cerr << "       " << basename(progname) << " -b trules preproc [input [output]]" << endl;
-  cerr << "       " << basename(progname) << " -n trules preproc [input [output]]" << endl;
-  cerr << "       " << basename(progname) << " -x extended trules preproc biltrans [input [output]]" << endl;
-  cerr << "       " << basename(progname) << " -c trules preproc biltrans [input [output]]" << endl;
-  cerr << "       " << basename(progname) << " -t trules preproc biltrans [input [output]]" << endl;
-  cerr << "  trules     transfer rules file" << endl;
-  cerr << "  preproc    result of preprocess trules file" << endl;
-  cerr << "  biltrans   bilingual letter transducer file" << endl;
-  cerr << "  input      input file, standard input by default" << endl;
-  cerr << "  output     output file, standard output by default" << endl;
-  cerr << "  -b         input from lexical transfer" << endl;
-  cerr << "  -n         don't use bilingual dictionary" << endl;
-  cerr << "  -x bindix  extended mode with user dictionary" << endl;
-  cerr << "  -c         case-sensitiveness while accessing bilingual dictionary" << endl;
-  cerr << "  -t         trace (show rule numbers and patterns matched)" << endl;
-  cerr << "  -T         trace, for apertium-transfer-tools (also sets -t)" << endl;
-  cerr << "  -z         null-flushing output on '\0'" << endl;
-  cerr << "  -h         shows this message" << endl;
+  wcerr << "USAGE: " << basename(progname) << " trules preproc biltrans [input [output]]" << endl;
+  wcerr << "       " << basename(progname) << " -b trules preproc [input [output]]" << endl;
+  wcerr << "       " << basename(progname) << " -n trules preproc [input [output]]" << endl;
+  wcerr << "       " << basename(progname) << " -x extended trules preproc biltrans [input [output]]" << endl;
+  wcerr << "       " << basename(progname) << " -c trules preproc biltrans [input [output]]" << endl;
+  wcerr << "       " << basename(progname) << " -t trules preproc biltrans [input [output]]" << endl;
+  wcerr << "  trules     transfer rules file" << endl;
+  wcerr << "  preproc    result of preprocess trules file" << endl;
+  wcerr << "  biltrans   bilingual letter transducer file" << endl;
+  wcerr << "  input      input file, standard input by default" << endl;
+  wcerr << "  output     output file, standard output by default" << endl;
+  wcerr << "  -b         input from lexical transfer" << endl;
+  wcerr << "  -n         don't use bilingual dictionary" << endl;
+  wcerr << "  -x bindix  extended mode with user dictionary" << endl;
+  wcerr << "  -c         case-sensitiveness while accessing bilingual dictionary" << endl;
+  wcerr << "  -t         trace (show rule numbers and patterns matched)" << endl;
+  wcerr << "  -T         trace, for apertium-transfer-tools (also sets -t)" << endl;
+  wcerr << "  -z         null-flushing output on '\0'" << endl;
+  wcerr << "  -h         shows this message" << endl;
   
 
   exit(EXIT_FAILURE);
@@ -64,8 +64,8 @@ void testfile(string const &filename)
   struct stat mybuf;
   if(stat(filename.c_str(), &mybuf) == -1)
   {
-    cerr << "Error: can't stat file '";
-    cerr << filename << "'." << endl;
+    wcerr << "Error: can't stat file '";
+    wcerr << filename << "'." << endl;
     exit(EXIT_FAILURE);
   }
 }
@@ -75,8 +75,8 @@ FILE * open_input(string const &filename)
   FILE *input = fopen(filename.c_str(), "r");
   if(!input)
   {
-    cerr << "Error: can't open input file '";
-    cerr << filename.c_str() << "'." << endl;
+    wcerr << "Error: can't open input file '";
+    wcerr << filename.c_str() << "'." << endl;
     exit(EXIT_FAILURE);
   }
   
@@ -88,8 +88,8 @@ FILE * open_output(string const &filename)
   FILE *output = fopen(filename.c_str(), "w");
   if(!output)
   {
-    cerr << "Error: can't open output file '";
-    cerr << filename.c_str() << "'." << endl;
+    wcerr << "Error: can't open output file '";
+    wcerr << filename.c_str() << "'." << endl;
     exit(EXIT_FAILURE);
   }
   return output;
@@ -101,12 +101,9 @@ int main(int argc, char *argv[])
  
   Transfer t;
 
-#if HAVE_GETOPT_LONG
   int option_index=0;
-#endif
 
   while (true) {
-#if HAVE_GETOPT_LONG
     static struct option long_options[] =
     {
       {"from-bilingual",      no_argument, 0, 'b'},
@@ -121,9 +118,6 @@ int main(int argc, char *argv[])
     };
 
     int c=getopt_long(argc, argv, "nbx:cztTh", long_options, &option_index);
-#else
-    int c=getopt(argc, argv, "nbx:cztTh");
-#endif
     if (c==-1)
       break;
       
