@@ -64,6 +64,12 @@ TaggerData::getOpenClass()
   return open_class;
 }
 
+const set<TTag> &
+TaggerData::getOpenClass() const
+{
+  return open_class;
+}
+
 void
 TaggerData::setOpenClass(set<TTag> const &oc)
 {
@@ -76,6 +82,12 @@ TaggerData::getForbidRules()
   return forbid_rules;
 }
 
+const vector<TForbidRule> &
+TaggerData::getForbidRules() const
+{
+  return forbid_rules;
+}
+
 void
 TaggerData::setForbidRules(vector<TForbidRule> &fr)
 {
@@ -88,6 +100,12 @@ TaggerData::getTagIndex()
   return tag_index;
 }
 
+const map<wstring, TTag, Ltstr> &
+TaggerData::getTagIndex() const
+{
+  return tag_index;
+}
+
 void
 TaggerData::setTagIndex(map<wstring, TTag, Ltstr> const &ti)
 {
@@ -100,6 +118,12 @@ TaggerData::getArrayTags()
   return array_tags;
 }
 
+const vector<wstring> &
+TaggerData::getArrayTags() const
+{
+  return array_tags;
+}
+
 void
 TaggerData::setArrayTags(vector<wstring> const &at)
 {
@@ -112,6 +136,12 @@ TaggerData::getEnforceRules()
   return enforce_rules;
 }
 
+const vector<TEnforceAfterRule> &
+TaggerData::getEnforceRules() const
+{
+  return enforce_rules;
+}
+
 void
 TaggerData::setEnforceRules(vector<TEnforceAfterRule> const &tear)
 {
@@ -124,6 +154,12 @@ TaggerData::getPreferRules()
   return prefer_rules;
 }
 
+const vector<wstring> &
+TaggerData::getPreferRules() const
+{
+  return prefer_rules;
+}
+
 void
 TaggerData::setPreferRules(vector<wstring> const &pr)
 {
@@ -136,6 +172,12 @@ TaggerData::getDiscardRules()
   return discard;
 }
 
+const vector<wstring> &
+TaggerData::getDiscardRules() const
+{
+  return discard;
+}
+
 void
 TaggerData::setDiscardRules(vector<wstring> const &v)
 {
@@ -160,6 +202,12 @@ TaggerData::getOutput()
   return output;
 }
 
+const Collection &
+TaggerData::getOutput() const
+{
+  return output;
+}
+
 void
 TaggerData::setOutput(Collection const &c)
 {
@@ -172,6 +220,12 @@ TaggerData::getPatternList()
   return plist;
 }
 
+const PatternList &
+TaggerData::getPatternList() const
+{
+  return plist;
+}
+
 void
 TaggerData::setPatternList(PatternList const &pl)
 {
