@@ -204,13 +204,13 @@ TaggerDataLSW::read(FILE *in)
   // read discards on ambiguity
   discard.clear();
 
-  int limit = Compression::multibyte_read(in);  
+  unsigned int limit = Compression::multibyte_read(in);  
   if(feof(in))
   {
     return;
   }
   
-  for(int i = 0; i < limit; i++)
+  for(unsigned int i = 0; i < limit; i++)
   {
     discard.push_back(Compression::wstring_read(in));
   }
@@ -309,7 +309,7 @@ TaggerDataLSW::write(FILE *out)
   
   // write pattern list
   plist.write(out);
-  
+
   // write discard list
   
   if(discard.size() != 0)
