@@ -16,9 +16,12 @@
  */
 
 #include <apertium/tagger_utils.h>
-#include <apertium/morpho_stream.h>
+#include <apertium/file_morpho_stream.h>
 
 #include <stdio.h>
+#include <sstream>
+#include <algorithm>
+#include <climits>
 #include <apertium/string_utils.h>
 #ifdef _MSC_VER
 #define wcstok wcstok_s
@@ -42,7 +45,7 @@ void tagger_utils::fatal_error (wstring const &s) {
 }
 
 void tagger_utils::file_name_error (string const &s) { 
-  cerr << "Error: " << s << endl;
+  wcerr << "Error: " << s << endl;
   exit(1);
 }
 
@@ -122,16 +125,18 @@ wstring tagger_utils::trim(wstring s)
   return s;
 }
 
-void
-tagger_utils::read_dictionary(FILE *fdic, TaggerData &td) {
-  int i, k, nw = 0;
-  TaggerWord *word = NULL;
-  set <TTag> tags;
+void tagger_utils::scan_for_ambg_classes(FILE *fdic, TaggerData &td) {
   Collection &output = td.getOutput();
+  FileMorphoStream morpho_stream(fdic, true, &td);
+  tagger_utils::scan_for_ambg_classes(output, morpho_stream);
+}
 
-  MorphoStream morpho_stream(fdic, true, &td);
+void tagger_utils::scan_for_ambg_classes(Collection &output, MorphoStream &morpho_stream) {
+  int nw = 0;
+  set <TTag> tags;
+  TaggerWord *word = NULL;
 
-  // In the input dictionary there must be all punctuation marks, including the end-of-sentece mark
+  // In the input dictionary there must be all punctuation marks, including the end-of-sentence mark
 
   word = morpho_stream.get_next_word();
 
@@ -142,17 +147,23 @@ tagger_utils::read_dictionary(FILE *fdic, TaggerData &td) {
     tags = word->get_tags();
 
     if (tags.size() > 0)
-      k = output[tags];
+      output[tags];
 
     delete word;
     word = morpho_stream.get_next_word();
   }
   wcerr << L"\n";
+}
+
+void
+tagger_utils::add_neccesary_ambg_classes(TaggerData &td) {
+  int i;
+  Collection &output = td.getOutput();
 
   // OPEN AMBIGUITY CLASS
   // It contains all tags that are not closed.
   // Unknown words are assigned the open ambiguity class
-  k = output[td.getOpenClass()];
+  output[td.getOpenClass()];
 
   // Create ambiguity class holding one single tag for each tag.
   // If not created yet
@@ -160,64 +171,82 @@ tagger_utils::read_dictionary(FILE *fdic, TaggerData &td) {
   for(i = 0; i != N; i++) {
     set<TTag> amb_class;
     amb_class.insert(i);
-    k = output[amb_class];
+    output[amb_class];
   }
 }
 
-set<TTag>
+set<TTag> &
 tagger_utils::find_similar_ambiguity_class(TaggerData &td, set<TTag> &c) {
-  int size_ret = -1;
-  set<TTag> ret = td.getOpenClass(); // return open-class as default, if no better is found.
-  bool skip_class;
+  set<TTag> &ret = td.getOpenClass();
   Collection &output = td.getOutput();
+  int ret_idx = output[ret];
 
-  for(int k=0; k<output.size(); k++) {
-    if ((((int)output[k].size())>((int)size_ret)) && (((int)output[k].size())<((int)c.size()))) {
-      skip_class = false;
-      // Test if output[k] is a subset of class
-      for(set<TTag>::const_iterator it=output[k].begin(); it!=output[k].end(); it++) {
-        if (c.find(*it)==c.end()) {
-           skip_class = true; //output[k] is not a subset of class
-           break;
-        }
-      }
-      if (!skip_class) {
-        size_ret = output[k].size();
-             ret = output[k];
-      }
+  for (int k=0; k<output.size(); k++) {
+    const set<TTag> &ambg_class = output[k];
+    if (ambg_class.size() > ret.size() ||
+        (ambg_class.size() == ret.size())) {
+      continue;
+    }
+    if (includes(ambg_class.begin(), ambg_class.end(), c.begin(), c.end())) {
+      ret_idx = k;
+      ret = ambg_class;
     }
   }
   return ret;
 }
 
 void
-tagger_utils::require_ambiguity_class(TaggerData &td, set<TTag> &tags, TaggerWord &word) {
+tagger_utils::require_ambiguity_class(TaggerData &td, set<TTag> &tags, TaggerWord &word, int nw) {
   if (td.getOutput().has_not(tags)) {
     wstring errors;
     errors = L"A new ambiguity class was found. I cannot continue.\n";
     errors+= L"Word '" + word.get_superficial_form() + L"' not found in the dictionary.\n";
     errors+= L"New ambiguity class: " + word.get_string_tags() + L"\n";
+    if (nw >= 0) {
+      std::wostringstream ws;
+      ws << (nw + 1);
+      errors+= L"Line number: " + ws.str() + L"\n";
+    }
     errors+= L"Take a look at the dictionary, then retrain.";
     fatal_error(errors);
   }
 }
 
-set<TTag>
-tagger_utils::require_similar_ambiguity_class(TaggerData &td, set<TTag> &tags, TaggerWord &word, bool debug) {
+static void _warn_absent_ambiguity_class(TaggerWord &word) {
+  wstring errors;
+  errors = L"A new ambiguity class was found. \n";
+  errors += L"Retraining the tagger is necessary so as to take it into account.\n";
+  errors += L"Word '" + word.get_superficial_form() + L"'.\n";
+  errors += L"New ambiguity class: " + word.get_string_tags() + L"\n";
+  wcerr << L"Error: " << errors;
+}
+
+set<TTag> &
+tagger_utils::require_similar_ambiguity_class(TaggerData &td, set<TTag> &tags, TaggerWord &word, bool warn) {
   if (td.getOutput().has_not(tags)) {
-    if (debug) {
-      wstring errors;
-      errors = L"A new ambiguity class was found. \n";
-      errors += L"Retraining the tagger is necessary so as to take it into account.\n";
-      errors += L"Word '" + word.get_superficial_form() + L"'.\n";
-      errors += L"New ambiguity class: " + word.get_string_tags() + L"\n";
-      wcerr << L"Error: " << errors;
+    if (warn) {
+      _warn_absent_ambiguity_class(word);
     }
     return find_similar_ambiguity_class(td, tags);
   }
   return tags;
 }
 
+set<TTag> &
+tagger_utils::require_similar_ambiguity_class(TaggerData &td, set<TTag> &tags) {
+  if (td.getOutput().has_not(tags)) {
+    return find_similar_ambiguity_class(td, tags);
+  }
+  return tags;
+}
+
+void
+tagger_utils::warn_absent_ambiguity_class(TaggerData &td, set<TTag> &tags, TaggerWord &word, bool warn) {
+  if (warn && td.getOutput().has_not(tags)) {
+    _warn_absent_ambiguity_class(word);
+  }
+}
+
 template <class T>
 ostream& operator<< (ostream& os, const map <int, T> & f){
   typename map <int, T>::const_iterator it;
