@@ -30,7 +30,8 @@ TransferData::copy(TransferData const &o)
 {
   alphabet = o.alphabet;
   transducer = o.transducer;
-  finals = o.finals;
+  final_symbols = o.final_symbols;
+  seen_rules = o.seen_rules;
   attr_items = o.attr_items;
   macros = o.macros;
   lists = o.lists;
@@ -45,9 +46,9 @@ TransferData::destroy()
 TransferData::TransferData()
 {
   // adding fixed attr_items
-  attr_items[L"lem"] = L"(([^<]|\"\\<\")+)";
+  attr_items[L"lem"] = L"^(([^<]|\"\\<\")+)";
   attr_items[L"lemq"] = L"\\#[- _][^<]+";
-  attr_items[L"lemh"] = L"(([^<#]|\"\\<\"|\"\\#\")+)";
+  attr_items[L"lemh"] = L"^(([^<#]|\"\\<\"|\"\\#\")+)";
   attr_items[L"whole"] = L"(.+)";
   attr_items[L"tags"] = L"((<[^>]+>)+)";
   attr_items[L"chname"] = L"({([^/]+)\\/)"; // includes delimiters { and / !!!
@@ -88,12 +89,6 @@ TransferData::getTransducer()
   return transducer;
 }
 
-map<int, int> &
-TransferData::getFinals()
-{
-  return finals;
-}
-
 map<wstring, wstring, Ltstr> &
 TransferData::getAttrItems()
 {
@@ -118,16 +113,67 @@ TransferData::getVariables()
   return variables;
 }
 
+int
+TransferData::countToFinalSymbol(const int count) {
+  const wstring count_sym = L"<RULE_NUMBER:" + to_wstring(count) + L">";
+  alphabet.includeSymbol(count_sym);
+  const int symbol = alphabet(count_sym);
+  final_symbols.insert(symbol);
+  return symbol;
+}
+
 void
 TransferData::write(FILE *output)
 {
   alphabet.write(output);
+
+  transducer.minimize();
+  set<int> old_finals = transducer.getFinals(); // copy for later removal
+  map<int, int> finals_rules;                   // node id -> rule number
+  map<int, multimap<int, int> >& transitions = transducer.getTransitions();
+  // Find all arcs with "final_symbols" in the transitions, let their source node instead be final,
+  // and extract the rule number from the arc. Record relation between source node and rule number
+  // in finals_rules. It is now no longer safe to minimize -- but we already did that.
+  const wstring rule_sym_pre = L"<RULE_NUMBER:"; // see countToFinalSymbol()
+  for(map<int, multimap<int, int> >::const_iterator it = transitions.begin(),
+        limit = transitions.end(); it != limit; ++it)
+  {
+    const int src = it->first;
+    for(multimap<int, int>::const_iterator arc = it->second.begin(),
+          arclimit = it->second.end(); arc != arclimit; ++arc)
+    {
+      const int symbol = arc->first;
+      const int trg = arc->second;
+      if(final_symbols.count(symbol) == 0) {
+        continue;
+      }
+      if(!transducer.isFinal(trg)) {
+        continue;
+      }
+      // Extract the rule number encoded by countToFinalSymbol():
+      wstring s;
+      alphabet.getSymbol(s, symbol);
+      if(s.compare(0, rule_sym_pre.size(), rule_sym_pre) != 0) {
+        continue;
+      }
+      const int rule_num = stoi(s.substr(rule_sym_pre.size()));
+      transducer.setFinal(src);
+      finals_rules[src] = rule_num;
+    }
+  }
+  // Remove the old finals:
+  for(set<int>::const_iterator it = old_finals.begin(), limit = old_finals.end();
+      it != limit; ++it)
+  {
+    transducer.setFinal(*it, false);
+  }
+
   transducer.write(output, alphabet.size());
 
-  // finals
+  // finals_rules
 
-  Compression::multibyte_write(finals.size(), output);
-  for(map<int, int>::const_iterator it = finals.begin(), limit = finals.end();
+  Compression::multibyte_write(finals_rules.size(), output);
+  for(map<int, int>::const_iterator it = finals_rules.begin(), limit = finals_rules.end();
       it != limit; it++)
   {
     Compression::multibyte_write(it->first, output);
